@@ -128,6 +128,15 @@ int arch_board_shutdown(void)
 	return VMM_EFAIL;
 }
 
+/*
+ * Print board information
+ */
+
+void arch_board_print_info(struct vmm_chardev *cdev)
+{
+	/* FIXME: To be implemented. */
+}
+
 /*
  * Initialization functions
  */
