@@ -272,6 +272,15 @@ struct clcd_board clcd_system_data = {
 	.remove		= versatile_clcd_remove,
 };
 
+/*
+ * Print board information
+ */
+
+void arch_board_print_info(struct vmm_chardev *cdev)
+{
+	/* FIXME: To be implemented. */
+}
+
 /*
  * Initialization functions
  */
