@@ -32,22 +32,23 @@
 #include <cpu_inline_asm.h>
 #include <cpu_vcpu_cp15.h>
 #include <cpu_vcpu_helper.h>
+#include <arm_features.h>
 
-void cpu_vcpu_halt(struct vmm_vcpu * vcpu, arch_regs_t * regs)
+void cpu_vcpu_halt(struct vmm_vcpu *vcpu, arch_regs_t *regs)
 {
 	if (!vcpu || !regs) {
 		return;
 	}
 
-	if (vcpu->state != VMM_VCPU_STATE_HALTED) {
+	if (vmm_manager_vcpu_get_state(vcpu) != VMM_VCPU_STATE_HALTED) {
 		vmm_printf("\n");
 		cpu_vcpu_dump_user_reg(vcpu, regs);
 		vmm_manager_vcpu_halt(vcpu);
 	}
 }
 
-u32 cpu_vcpu_cpsr_retrieve(struct vmm_vcpu * vcpu,
-			  arch_regs_t * regs)
+u32 cpu_vcpu_cpsr_retrieve(struct vmm_vcpu *vcpu,
+			  arch_regs_t *regs)
 {
 	if (!vcpu || !regs) {
 		return 0;
@@ -60,7 +61,7 @@ u32 cpu_vcpu_cpsr_retrieve(struct vmm_vcpu * vcpu,
 	}
 }
 
-void cpu_vcpu_banked_regs_save(struct vmm_vcpu * vcpu, arch_regs_t * src)
+void cpu_vcpu_banked_regs_save(struct vmm_vcpu *vcpu, arch_regs_t *src)
 {
 	if (!vcpu || !vcpu->is_normal || !src) {
 		return;
@@ -143,7 +144,7 @@ void cpu_vcpu_banked_regs_save(struct vmm_vcpu * vcpu, arch_regs_t * src)
 	};
 }
 
-void cpu_vcpu_banked_regs_restore(struct vmm_vcpu * vcpu, arch_regs_t * dst)
+void cpu_vcpu_banked_regs_restore(struct vmm_vcpu *vcpu, arch_regs_t *dst)
 {
 	if (!vcpu || !vcpu->is_normal || !dst) {
 		return;
@@ -226,15 +227,15 @@ void cpu_vcpu_banked_regs_restore(struct vmm_vcpu * vcpu, arch_regs_t * dst)
 	};
 }
 
-void cpu_vcpu_cpsr_update(struct vmm_vcpu * vcpu, 
-			  arch_regs_t * regs,
+void cpu_vcpu_cpsr_update(struct vmm_vcpu *vcpu, 
+			  arch_regs_t *regs,
 			  u32 new_cpsr,
 			  u32 new_cpsr_mask)
 {
 	bool mode_change;
 
 	/* Sanity check */
-	if (!vcpu && !vcpu->is_normal && !regs) {
+	if (!vcpu || !vcpu->is_normal || !regs) {
 		return;
 	}
 	new_cpsr &= new_cpsr_mask;
@@ -263,7 +264,7 @@ void cpu_vcpu_cpsr_update(struct vmm_vcpu * vcpu,
 	return;
 }
 
-u32 cpu_vcpu_spsr_retrieve(struct vmm_vcpu * vcpu)
+u32 cpu_vcpu_spsr_retrieve(struct vmm_vcpu *vcpu)
 {
 	/* Find out correct SPSR */
 	switch (arm_priv(vcpu)->cpsr & CPSR_MODE_MASK) {
@@ -291,12 +292,12 @@ u32 cpu_vcpu_spsr_retrieve(struct vmm_vcpu * vcpu)
 	return 0;
 }
 
-int cpu_vcpu_spsr_update(struct vmm_vcpu * vcpu, 
+int cpu_vcpu_spsr_update(struct vmm_vcpu *vcpu, 
 			 u32 new_spsr,
 			 u32 new_spsr_mask)
 {
 	/* Sanity check */
-	if (!vcpu && !vcpu->is_normal) {
+	if (!vcpu || !vcpu->is_normal) {
 		return VMM_EFAIL;
 	}
 	if ((arm_priv(vcpu)->cpsr & CPSR_MODE_MASK) == CPSR_MODE_USER) {
@@ -336,8 +337,8 @@ int cpu_vcpu_spsr_update(struct vmm_vcpu * vcpu,
 	return VMM_OK;
 }
 
-u32 cpu_vcpu_reg_read(struct vmm_vcpu * vcpu, 
-		      arch_regs_t * regs, 
+u32 cpu_vcpu_reg_read(struct vmm_vcpu *vcpu, 
+		      arch_regs_t *regs, 
 		      u32 reg_num) 
 {
 	switch (reg_num) {
@@ -371,8 +372,8 @@ u32 cpu_vcpu_reg_read(struct vmm_vcpu * vcpu,
 	return 0x0;
 }
 
-void cpu_vcpu_reg_write(struct vmm_vcpu * vcpu, 
-			arch_regs_t * regs, 
+void cpu_vcpu_reg_write(struct vmm_vcpu *vcpu, 
+			arch_regs_t *regs, 
 			u32 reg_num, 
 			u32 reg_val) 
 {
@@ -466,8 +467,8 @@ void cpu_vcpu_reg_write(struct vmm_vcpu * vcpu,
 	};
 }
 
-u32 cpu_vcpu_regmode_read(struct vmm_vcpu * vcpu, 
-			  arch_regs_t * regs, 
+u32 cpu_vcpu_regmode_read(struct vmm_vcpu *vcpu, 
+			  arch_regs_t *regs, 
 			  u32 mode,
 			  u32 reg_num)
 {
@@ -568,8 +569,8 @@ u32 cpu_vcpu_regmode_read(struct vmm_vcpu * vcpu,
 	return 0x0;
 }
 
-void cpu_vcpu_regmode_write(struct vmm_vcpu * vcpu, 
-			    arch_regs_t * regs, 
+void cpu_vcpu_regmode_write(struct vmm_vcpu *vcpu, 
+			    arch_regs_t *regs, 
 			    u32 mode,
 			    u32 reg_num,
 			    u32 reg_val)
@@ -670,7 +671,7 @@ void cpu_vcpu_regmode_write(struct vmm_vcpu * vcpu,
 	}
 }
 
-int arch_guest_init(struct vmm_guest * guest)
+int arch_guest_init(struct vmm_guest *guest)
 {
 	int rc;
 	u32 ovect_flags;
@@ -715,39 +716,42 @@ int arch_guest_init(struct vmm_guest * guest)
 	return VMM_OK;
 }
 
-int arch_guest_deinit(struct vmm_guest * guest)
+int arch_guest_deinit(struct vmm_guest *guest)
 {
 	int rc;
-	if (!arm_guest_priv(guest)->ovect) {
-		rc = vmm_host_free_pages((virtual_addr_t)arm_guest_priv(guest)->ovect, 1);
-		if (rc) {
-			return rc;
-		}
-	}
 	if (guest->arch_priv) {
+		if (arm_guest_priv(guest)->ovect) {
+			rc = vmm_host_free_pages(
+			     (virtual_addr_t)arm_guest_priv(guest)->ovect, 1);
+			if (rc) {
+				return rc;
+			}
+		}
 		vmm_free(guest->arch_priv);
 	}
 	return VMM_OK;
 }
 
-int arch_vcpu_init(struct vmm_vcpu * vcpu)
+int arch_vcpu_init(struct vmm_vcpu *vcpu)
 {
 	u32 ite, cpuid;
-	const char * attr;
+	const char *attr;
 
 	/* Initialize User Mode Registers */
 	/* For both Orphan & Normal VCPUs */
 	memset(arm_regs(vcpu), 0, sizeof(arch_regs_t));
 	arm_regs(vcpu)->pc = vcpu->start_pc;
+	arm_regs(vcpu)->sp_excp = vcpu->stack_va + vcpu->stack_sz - 4;
 	if (vcpu->is_normal) {
 		arm_regs(vcpu)->cpsr  = CPSR_ZERO_MASK;
 		arm_regs(vcpu)->cpsr |= CPSR_ASYNC_ABORT_DISABLED;
 		arm_regs(vcpu)->cpsr |= CPSR_MODE_USER;
+		arm_regs(vcpu)->sp = 0;
 	} else {
 		arm_regs(vcpu)->cpsr  = CPSR_ZERO_MASK;
 		arm_regs(vcpu)->cpsr |= CPSR_ASYNC_ABORT_DISABLED;
 		arm_regs(vcpu)->cpsr |= CPSR_MODE_SUPERVISOR;
-		arm_regs(vcpu)->sp = vcpu->start_sp;
+		arm_regs(vcpu)->sp = arm_regs(vcpu)->sp_excp;
 	}
 	/* Initialize Supervisor Mode Registers */
 	/* For only Normal VCPUs */
@@ -756,18 +760,22 @@ int arch_vcpu_init(struct vmm_vcpu * vcpu)
 	}
 	attr = vmm_devtree_attrval(vcpu->node, 
 				   VMM_DEVTREE_COMPATIBLE_ATTR_NAME);
-	if (strcmp(attr, "ARMv7a,cortex-a8") == 0) {
-		cpuid = ARM_CPUID_CORTEXA8;
-	} else if (strcmp(attr, "ARMv7a,cortex-a9") == 0) {
-		cpuid = ARM_CPUID_CORTEXA9;
-	} else if (strcmp(attr, "ARMv5te,ARM926ej") == 0) {
+	if (!attr) {
+		return VMM_EFAIL;
+	}
+	if (strcmp(attr, "armv5te,arm926ej") == 0) {
 		cpuid = ARM_CPUID_ARM926;
+	} else if (strcmp(attr, "armv6,arm11mp") == 0) {
+		cpuid = ARM_CPUID_ARM11MPCORE;
+	} else if (strcmp(attr, "armv7a,cortex-a8") == 0) {
+		cpuid = ARM_CPUID_CORTEXA8;
+	} else if (strcmp(attr, "armv7a,cortex-a9") == 0) {
+		cpuid = ARM_CPUID_CORTEXA9;
 	} else {
 		return VMM_EFAIL;
 	}
 	if (!vcpu->reset_count) {
-		vcpu->arch_priv = vmm_malloc(sizeof(arm_priv_t));
-		memset(arm_priv(vcpu), 0, sizeof(arm_priv_t));
+		vcpu->arch_priv = vmm_zalloc(sizeof(arm_priv_t));
 		arm_priv(vcpu)->cpsr = CPSR_ASYNC_ABORT_DISABLED | 
 				   CPSR_IRQ_DISABLED |
 				   CPSR_FIQ_DISABLED | 
@@ -810,47 +818,84 @@ int arch_vcpu_init(struct vmm_vcpu * vcpu)
 		arm_priv(vcpu)->features = 0;
 		switch (cpuid) {
 		case ARM_CPUID_ARM926:
-			arm_set_feature(vcpu, ARM_FEATURE_V4T);
 			arm_set_feature(vcpu, ARM_FEATURE_V5);
 			arm_set_feature(vcpu, ARM_FEATURE_VFP);
+			arm_set_feature(vcpu, ARM_FEATURE_DUMMY_C15_REGS);
+			arm_set_feature(vcpu, ARM_FEATURE_CACHE_TEST_CLEAN);
 			break;
-		case ARM_CPUID_CORTEXA8:
-			arm_set_feature(vcpu, ARM_FEATURE_V4T);
-			arm_set_feature(vcpu, ARM_FEATURE_V5);
+		case ARM_CPUID_ARM11MPCORE:
 			arm_set_feature(vcpu, ARM_FEATURE_V6);
 			arm_set_feature(vcpu, ARM_FEATURE_V6K);
-			arm_set_feature(vcpu, ARM_FEATURE_V7);
-			arm_set_feature(vcpu, ARM_FEATURE_AUXCR);
-			arm_set_feature(vcpu, ARM_FEATURE_THUMB2);
 			arm_set_feature(vcpu, ARM_FEATURE_VFP);
+			arm_set_feature(vcpu, ARM_FEATURE_VAPA);
+			arm_set_feature(vcpu, ARM_FEATURE_DUMMY_C15_REGS);
+			break;
+		case ARM_CPUID_CORTEXA8:
+			arm_set_feature(vcpu, ARM_FEATURE_V7);
 			arm_set_feature(vcpu, ARM_FEATURE_VFP3);
 			arm_set_feature(vcpu, ARM_FEATURE_NEON);
 			arm_set_feature(vcpu, ARM_FEATURE_THUMB2EE);
+			arm_set_feature(vcpu, ARM_FEATURE_DUMMY_C15_REGS);
+			arm_set_feature(vcpu, ARM_FEATURE_TRUSTZONE);
 			break;
 		case ARM_CPUID_CORTEXA9:
-			arm_set_feature(vcpu, ARM_FEATURE_V4T);
-			arm_set_feature(vcpu, ARM_FEATURE_V5);
-			arm_set_feature(vcpu, ARM_FEATURE_V6);
-			arm_set_feature(vcpu, ARM_FEATURE_V6K);
 			arm_set_feature(vcpu, ARM_FEATURE_V7);
-			arm_set_feature(vcpu, ARM_FEATURE_AUXCR);
-			arm_set_feature(vcpu, ARM_FEATURE_THUMB2);
-			arm_set_feature(vcpu, ARM_FEATURE_VFP);
 			arm_set_feature(vcpu, ARM_FEATURE_VFP3);
 			arm_set_feature(vcpu, ARM_FEATURE_VFP_FP16);
 			arm_set_feature(vcpu, ARM_FEATURE_NEON);
 			arm_set_feature(vcpu, ARM_FEATURE_THUMB2EE);
 			arm_set_feature(vcpu, ARM_FEATURE_V7MP);
+			arm_set_feature(vcpu, ARM_FEATURE_TRUSTZONE);
 			break;
 		default:
 			break;
 		};
+
+		/* Some features automatically imply others: */
+		if (arm_feature(vcpu, ARM_FEATURE_V7)) {
+			arm_set_feature(vcpu, ARM_FEATURE_VAPA);
+			arm_set_feature(vcpu, ARM_FEATURE_THUMB2);
+			arm_set_feature(vcpu, ARM_FEATURE_MPIDR);
+			if (!arm_feature(vcpu, ARM_FEATURE_M)) {
+				arm_set_feature(vcpu, ARM_FEATURE_V6K);
+			} else {
+				arm_set_feature(vcpu, ARM_FEATURE_V6);
+			}
+		}
+		if (arm_feature(vcpu, ARM_FEATURE_V6K)) {
+			arm_set_feature(vcpu, ARM_FEATURE_V6);
+			arm_set_feature(vcpu, ARM_FEATURE_MVFR);
+		}
+		if (arm_feature(vcpu, ARM_FEATURE_V6)) {
+			arm_set_feature(vcpu, ARM_FEATURE_V5);
+			if (!arm_feature(vcpu, ARM_FEATURE_M)) {
+				arm_set_feature(vcpu, ARM_FEATURE_AUXCR);
+			}
+		}
+		if (arm_feature(vcpu, ARM_FEATURE_V5)) {
+			arm_set_feature(vcpu, ARM_FEATURE_V4T);
+		}
+		if (arm_feature(vcpu, ARM_FEATURE_M)) {
+			arm_set_feature(vcpu, ARM_FEATURE_THUMB_DIV);
+		}
+		if (arm_feature(vcpu, ARM_FEATURE_ARM_DIV)) {
+			arm_set_feature(vcpu, ARM_FEATURE_THUMB_DIV);
+		}
+		if (arm_feature(vcpu, ARM_FEATURE_VFP4)) {
+			arm_set_feature(vcpu, ARM_FEATURE_VFP3);
+		}
+		if (arm_feature(vcpu, ARM_FEATURE_VFP3)) {
+			arm_set_feature(vcpu, ARM_FEATURE_VFP);
+		}
+		if (arm_feature(vcpu, ARM_FEATURE_LPAE)) {
+			arm_set_feature(vcpu, ARM_FEATURE_PXN);
+		}
 	}
 
 	return cpu_vcpu_cp15_init(vcpu, cpuid);
 }
 
-int arch_vcpu_deinit(struct vmm_vcpu * vcpu)
+int arch_vcpu_deinit(struct vmm_vcpu *vcpu)
 {
 	int rc;
 
@@ -873,9 +918,9 @@ int arch_vcpu_deinit(struct vmm_vcpu * vcpu)
 	return VMM_OK;
 }
 
-void arch_vcpu_switch(struct vmm_vcpu * tvcpu,
-		      struct vmm_vcpu * vcpu, 
-                      arch_regs_t * regs)
+void arch_vcpu_switch(struct vmm_vcpu *tvcpu,
+		      struct vmm_vcpu *vcpu, 
+                      arch_regs_t *regs)
 {
 	u32 ite;
 	/* Save user registers & banked registers */
@@ -887,6 +932,7 @@ void arch_vcpu_switch(struct vmm_vcpu * tvcpu,
 			arm_regs(tvcpu)->gpr[ite] = regs->gpr[ite];
 		}
 		arm_regs(tvcpu)->cpsr = regs->cpsr;
+		arm_regs(tvcpu)->sp_excp = regs->sp_excp;
 		if(tvcpu->is_normal) {
 			cpu_vcpu_banked_regs_save(tvcpu, regs);
 		}
@@ -901,6 +947,7 @@ void arch_vcpu_switch(struct vmm_vcpu * tvcpu,
 		regs->gpr[ite] = arm_regs(vcpu)->gpr[ite];
 	}
 	regs->cpsr = arm_regs(vcpu)->cpsr;
+	regs->sp_excp = arm_regs(vcpu)->sp_excp;
 	if (vcpu->is_normal) {
 		cpu_vcpu_banked_regs_restore(vcpu, regs);
 	}
@@ -908,69 +955,86 @@ void arch_vcpu_switch(struct vmm_vcpu * tvcpu,
 	clrex();
 }
 
-void cpu_vcpu_dump_user_reg(struct vmm_vcpu * vcpu, arch_regs_t * regs)
+void arch_vcpu_preempt_orphan(void)
 {
-	u32 ite;
-	vmm_printf("  Core Registers\n");
-	vmm_printf("    SP=0x%08x       LR=0x%08x       PC=0x%08x\n",
-		   regs->sp, regs->lr, regs->pc);
-	vmm_printf("    CPSR=0x%08x     \n", 
-				cpu_vcpu_cpsr_retrieve(vcpu, regs));
-	vmm_printf("  General Purpose Registers");
-	for (ite = 0; ite < CPU_GPR_COUNT; ite++) {
-		if (ite % 3 == 0)
-			vmm_printf("\n");
-		vmm_printf("    R%02d=0x%08x  ", ite, regs->gpr[ite]);
-	}
-	vmm_printf("\n");
+	/* Trigger SVC call from supervisor mode. This will cause
+	 * do_soft_irq() function to call vmm_scheduler_preempt_orphan()
+	 */
+	asm volatile ("svc #0\t\n");
 }
 
-void arch_vcpu_regs_dump(struct vmm_vcpu * vcpu)
+static void __cpu_vcpu_dump_user_reg(struct vmm_chardev *cdev, 
+				     struct vmm_vcpu *vcpu, arch_regs_t *regs)
 {
 	u32 ite;
+	vmm_cprintf(cdev, "  Core Registers\n");
+	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       PC=0x%08x\n",
+		    regs->sp, regs->lr, regs->pc);
+	vmm_cprintf(cdev, "    CPSR=0x%08x     \n", 
+		    cpu_vcpu_cpsr_retrieve(vcpu, regs));
+	vmm_cprintf(cdev, "  General Purpose Registers");
+	for (ite = 0; ite < CPU_GPR_COUNT; ite++) {
+		if (ite % 3 == 0)
+			vmm_cprintf(cdev, "\n");
+		vmm_cprintf(cdev, "    R%02d=0x%08x  ", ite, regs->gpr[ite]);
+	}
+	vmm_cprintf(cdev, "\n");
+}
+
+void cpu_vcpu_dump_user_reg(struct vmm_vcpu *vcpu, arch_regs_t *regs)
+{
+	__cpu_vcpu_dump_user_reg(NULL, vcpu, regs);
+}
+
+void arch_vcpu_regs_dump(struct vmm_chardev *cdev, struct vmm_vcpu *vcpu)
+{
+	u32 ite;
+
 	/* For both Normal & Orphan VCPUs */
-	cpu_vcpu_dump_user_reg(vcpu, arm_regs(vcpu));
+	__cpu_vcpu_dump_user_reg(cdev, vcpu, arm_regs(vcpu));
+
 	/* For only Normal VCPUs */
 	if (!vcpu->is_normal) {
 		return;
 	}
-	vmm_printf("  User Mode Registers (Banked)\n");
-	vmm_printf("    SP=0x%08x       LR=0x%08x\n",
-		   arm_priv(vcpu)->sp_usr, arm_priv(vcpu)->lr_usr);
-	vmm_printf("  Supervisor Mode Registers (Banked)\n");
-	vmm_printf("    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
-		   arm_priv(vcpu)->sp_svc, arm_priv(vcpu)->lr_svc,
-		   arm_priv(vcpu)->spsr_svc);
-	vmm_printf("  Monitor Mode Registers (Banked)\n");
-	vmm_printf("    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
-		   arm_priv(vcpu)->sp_mon, arm_priv(vcpu)->lr_mon,
-		   arm_priv(vcpu)->spsr_mon);
-	vmm_printf("  Abort Mode Registers (Banked)\n");
-	vmm_printf("    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
-		   arm_priv(vcpu)->sp_abt, arm_priv(vcpu)->lr_abt,
-		   arm_priv(vcpu)->spsr_abt);
-	vmm_printf("  Undefined Mode Registers (Banked)\n");
-	vmm_printf("    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
-		   arm_priv(vcpu)->sp_und, arm_priv(vcpu)->lr_und,
-		   arm_priv(vcpu)->spsr_und);
-	vmm_printf("  IRQ Mode Registers (Banked)\n");
-	vmm_printf("    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
-		   arm_priv(vcpu)->sp_irq, arm_priv(vcpu)->lr_irq,
-		   arm_priv(vcpu)->spsr_irq);
-	vmm_printf("  FIQ Mode Registers (Banked)\n");
-	vmm_printf("    SP=0x%08x       LR=0x%08x       SPSR=0x%08x",
-		   arm_priv(vcpu)->sp_fiq, arm_priv(vcpu)->lr_fiq,
-		   arm_priv(vcpu)->spsr_fiq);
+
+	vmm_cprintf(cdev, "  User Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x\n",
+		    arm_priv(vcpu)->sp_usr, arm_priv(vcpu)->lr_usr);
+	vmm_cprintf(cdev, "  Supervisor Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
+		    arm_priv(vcpu)->sp_svc, arm_priv(vcpu)->lr_svc,
+		    arm_priv(vcpu)->spsr_svc);
+	vmm_cprintf(cdev, "  Monitor Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
+		    arm_priv(vcpu)->sp_mon, arm_priv(vcpu)->lr_mon,
+		    arm_priv(vcpu)->spsr_mon);
+	vmm_cprintf(cdev, "  Abort Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
+		    arm_priv(vcpu)->sp_abt, arm_priv(vcpu)->lr_abt,
+		    arm_priv(vcpu)->spsr_abt);
+	vmm_cprintf(cdev, "  Undefined Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
+		    arm_priv(vcpu)->sp_und, arm_priv(vcpu)->lr_und,
+		    arm_priv(vcpu)->spsr_und);
+	vmm_cprintf(cdev, "  IRQ Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x\n",
+		    arm_priv(vcpu)->sp_irq, arm_priv(vcpu)->lr_irq,
+		    arm_priv(vcpu)->spsr_irq);
+	vmm_cprintf(cdev, "  FIQ Mode Registers (Banked)\n");
+	vmm_cprintf(cdev, "    SP=0x%08x       LR=0x%08x       SPSR=0x%08x",
+		    arm_priv(vcpu)->sp_fiq, arm_priv(vcpu)->lr_fiq,
+		    arm_priv(vcpu)->spsr_fiq);
 	for (ite = 0; ite < 5; ite++) {
 		if (ite % 3 == 0)
-			vmm_printf("\n");
-		vmm_printf("    R%02d=0x%08x  ", (ite + 8),
-			   arm_priv(vcpu)->gpr_fiq[ite]);
+			vmm_cprintf(cdev, "\n");
+		vmm_cprintf(cdev, "    R%02d=0x%08x  ", (ite + 8),
+			    arm_priv(vcpu)->gpr_fiq[ite]);
 	}
-	vmm_printf("\n");
+	vmm_cprintf(cdev, "\n");
 }
 
-void arch_vcpu_stat_dump(struct vmm_vcpu * vcpu)
+void arch_vcpu_stat_dump(struct vmm_chardev *cdev, struct vmm_vcpu *vcpu)
 {
-	vmm_printf("No VCPU stats available.\n");
+	/* For now no arch specific stats */
 }
