@@ -561,6 +561,18 @@
 #define PAR64_F_MASK					0x0000000000000001ULL
 #define PAR64_F_SHIFT					0
 
+/* MIDR */
+#define MIDR_IMPLEMENTER_MASK				0xFF000000
+#define MIDR_IMPLEMENTER_SHIFT				24
+#define MIDR_VARIANT_MASK				0x00F00000
+#define MIDR_VARIANT_SHIFT				20
+#define MIDR_ARCHITECTURE_MASK				0x000F0000
+#define MIDR_ARCHITECTURE_SHIFT				16
+#define MIDR_PARTNUM_MASK				0x0000FFF0
+#define MIDR_PARTNUM_SHIFT				4
+#define MIDR_REVISON_MASK				0x0000000F
+#define MIDR_REVISON_SHIFT				0
+
 /* ID_PFR1 */
 #define ID_PFR1_GEN_TIMER_MASK				0x000f0000
 #define ID_PFR1_GEN_TIMER_SHIFT				16
