@@ -17,57 +17,64 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * @file cmd_chardev.c
- * @version 0.01
  * @author Anup Patel (anup@brainfault.org)
  * @brief Implementation of chardev command
  */
 
 #include <vmm_error.h>
 #include <vmm_stdio.h>
-#include <vmm_string.h>
 #include <vmm_devtree.h>
 #include <vmm_chardev.h>
 #include <vmm_modules.h>
 #include <vmm_cmdmgr.h>
+#include <libs/stringlib.h>
 
-#define MODULE_VARID			cmd_chardev_module
-#define MODULE_NAME			"Command chardev"
+#define MODULE_DESC			"Command chardev"
 #define MODULE_AUTHOR			"Anup Patel"
+#define MODULE_LICENSE			"GPL"
 #define MODULE_IPRIORITY		0
 #define	MODULE_INIT			cmd_chardev_init
 #define	MODULE_EXIT			cmd_chardev_exit
 
-void cmd_chardev_usage(vmm_chardev_t *cdev)
+void cmd_chardev_usage(struct vmm_chardev *cdev)
 {
 	vmm_cprintf(cdev, "Usage:\n");
 	vmm_cprintf(cdev, "   chardev help\n");
 	vmm_cprintf(cdev, "   chardev list\n");
 }
 
-void cmd_chardev_list(vmm_chardev_t *cdev)
+void cmd_chardev_list(struct vmm_chardev *cdev)
 {
 	int num, count;
 	char path[1024];
-	vmm_chardev_t *cd;
+	struct vmm_chardev *cd;
+	vmm_cprintf(cdev, "----------------------------------------"
+			  "----------------------------------------\n");
+	vmm_cprintf(cdev, " %-24s %-53s\n", 
+			  "Name", "Device Path");
+	vmm_cprintf(cdev, "----------------------------------------"
+			  "----------------------------------------\n");
 	count = vmm_chardev_count();
 	for (num = 0; num < count; num++) {
 		cd = vmm_chardev_get(num);
 		if (!cd->dev) {
-			vmm_cprintf(cdev, "%s: ---\n", cd->name);
+			strcpy(path, "-----");
 		} else {
 			vmm_devtree_getpath(path, cd->dev->node);
-			vmm_cprintf(cdev, "%s: %s\n", cd->name, path);
 		}
+		vmm_cprintf(cdev, " %-24s %-53s\n", cd->name, path);
 	}
+	vmm_cprintf(cdev, "----------------------------------------"
+			  "----------------------------------------\n");
 }
 
-int cmd_chardev_exec(vmm_chardev_t *cdev, int argc, char **argv)
+int cmd_chardev_exec(struct vmm_chardev *cdev, int argc, char **argv)
 {
 	if (argc == 2) {
-		if (vmm_strcmp(argv[1], "help") == 0) {
+		if (strcmp(argv[1], "help") == 0) {
 			cmd_chardev_usage(cdev);
 			return VMM_OK;
-		} else if (vmm_strcmp(argv[1], "list") == 0) {
+		} else if (strcmp(argv[1], "list") == 0) {
 			cmd_chardev_list(cdev);
 			return VMM_OK;
 		}
@@ -79,7 +86,7 @@ int cmd_chardev_exec(vmm_chardev_t *cdev, int argc, char **argv)
 	return VMM_OK;
 }
 
-static vmm_cmd_t cmd_chardev = {
+static struct vmm_cmd cmd_chardev = {
 	.name = "chardev",
 	.desc = "character device commands",
 	.usage = cmd_chardev_usage,
@@ -91,14 +98,14 @@ static int __init cmd_chardev_init(void)
 	return vmm_cmdmgr_register_cmd(&cmd_chardev);
 }
 
-static void cmd_chardev_exit(void)
+static void __exit cmd_chardev_exit(void)
 {
 	vmm_cmdmgr_unregister_cmd(&cmd_chardev);
 }
 
-VMM_DECLARE_MODULE(MODULE_VARID, 
-			MODULE_NAME, 
+VMM_DECLARE_MODULE(MODULE_DESC, 
 			MODULE_AUTHOR, 
+			MODULE_LICENSE, 
 			MODULE_IPRIORITY, 
 			MODULE_INIT, 
 			MODULE_EXIT);
