@@ -26,6 +26,7 @@
 #include <vmm_types.h>
 #include <vmm_compiler.h>
 #include <libs/kallsyms.h>
+#include <libs/list.h>
 
 #define VMM_MODULE_SIGNATURE		0x564D4F44
 
@@ -61,15 +62,18 @@ typedef void (*vmm_module_exit_t) (void);
  * 3.	So vendors can do likewise based on their own policies
  */
 
+#include <vmm_limits.h>
+
 struct vmm_module {
 	u32 signature;
-	char name[48];
-	char desc[128];
-	char author[32];
-	char license[32];
+	char name[VMM_FIELD_NAME_SIZE];
+	char desc[VMM_FIELD_DESC_SIZE];
+	char author[VMM_FIELD_AUTHOR_SIZE];
+	char license[VMM_FIELD_LICENSE_SIZE];
 	u32 ipriority;
 	vmm_module_init_t init;
 	vmm_module_exit_t exit;
+	struct dlist head;
 };
 
 enum vmm_symbol_types {
@@ -88,21 +92,42 @@ struct vmm_symbol {
 
 #ifdef __VMM_MODULES__
 
-#define VMM_DECLARE_MODULE(desc,author,license,ipriority,init,exit) \
-static __unused __modtbl struct vmm_module __moddecl__ = \
-{ VMM_MODULE_SIGNATURE, VMM_MODNAME, desc, author, license, ipriority, init, exit }
+#define VMM_DECLARE_MODULE(_desc,_author,_license,_ipriority,_init,_exit) \
+static __unused __modtbl struct vmm_module __moddecl__ = {\
+	.signature = VMM_MODULE_SIGNATURE, \
+	.name = VMM_MODNAME, \
+	.desc = _desc, \
+	.author = _author, \
+	.license = _license, \
+	.ipriority = _ipriority, \
+	.init = _init, \
+	.exit = _exit, \
+	.head = LIST_HEAD_INIT(__moddecl__.head), \
+}
 
-#define __VMM_EXPORT_SYMBOL(sym,type) \
-static __unused __symtbl struct vmm_symbol __##sym = \
-{ #sym, (virtual_addr_t)&sym, (type) }
+#define __VMM_EXPORT_SYMBOL(sym,_type) \
+static __unused __symtbl struct vmm_symbol __##sym = { \
+	.name = #sym, \
+	.addr = (virtual_addr_t)&sym, \
+	.type = (_type), \
+}
 
 #else
 
-#define VMM_DECLARE_MODULE(desc,author,license,ipriority,init,exit) \
-static __unused __modtbl struct vmm_module __moddecl__ = \
-{ VMM_MODULE_SIGNATURE, VMM_MODNAME, desc, author, "GPL", ipriority, init, exit }
+#define VMM_DECLARE_MODULE(_desc,_author,_license,_ipriority,_init,_exit) \
+static __unused __modtbl struct vmm_module __moddecl__ = {\
+	.signature = VMM_MODULE_SIGNATURE, \
+	.name = VMM_MODNAME, \
+	.desc = _desc, \
+	.author = _author, \
+	.license = "GPL", \
+	.ipriority = _ipriority, \
+	.init = _init, \
+	.exit = _exit, \
+	.head = LIST_HEAD_INIT(__moddecl__.head), \
+}
 
-#define __VMM_EXPORT_SYMBOL(sym,type)
+#define __VMM_EXPORT_SYMBOL(sym,_type)
 
 #endif
 
