@@ -17,7 +17,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * @file vmm_vcpu_irq.h
- * @version 1.0
  * @author Anup Patel (anup@brainfault.org)
  * @brief header file for vcpu interrupts
  */
@@ -27,16 +26,31 @@
 #include <vmm_types.h>
 #include <vmm_manager.h>
 
-/** Process interrupts for current vcpu */
-void vmm_vcpu_irq_process(vmm_user_regs_t * regs);
+/** Process interrupts for current vcpu 
+ *  Note: Don't call this function directly it's meant to be called
+ *  from vmm_scheduler only.
+ */
+void vmm_vcpu_irq_process(struct vmm_vcpu *vcpu, arch_regs_t *regs);
 
 /** Assert an irq to given vcpu */
-void vmm_vcpu_irq_assert(vmm_vcpu_t *vcpu, u32 irq_no, u32 reason);
+void vmm_vcpu_irq_assert(struct vmm_vcpu *vcpu, u32 irq_no, u64 reason);
 
 /** Deassert active irq of given vcpu */
-void vmm_vcpu_irq_deassert(vmm_vcpu_t *vcpu);
+void vmm_vcpu_irq_deassert(struct vmm_vcpu *vcpu, u32 irq_no);
 
-/** Intialize interrupts for given vcpu */
-int vmm_vcpu_irq_init(vmm_vcpu_t *vcpu);
+/** Forcefully resume given VCPU if waiting for irq */
+int vmm_vcpu_irq_wait_resume(struct vmm_vcpu *vcpu);
+
+/** Wait for irq on given vcpu with some timeout */
+int vmm_vcpu_irq_wait_timeout(struct vmm_vcpu *vcpu, u64 nsecs);
+
+/** Wait for irq on given vcpu indefinetly (no timeout) */
+#define vmm_vcpu_irq_wait(vcpu)	vmm_vcpu_irq_wait_timeout(vcpu, 0)
+
+/** Initialize interrupts for given vcpu */
+int vmm_vcpu_irq_init(struct vmm_vcpu *vcpu);
+
+/** Deinitialize interrupts for given vcpu */
+int vmm_vcpu_irq_deinit(struct vmm_vcpu *vcpu);
 
 #endif
