@@ -17,16 +17,15 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
  * @file vmm_cmdmgr.c
- * @version 0.01
  * @author Anup Patel (anup@brainfault.org)
  * @brief source file of command manager
  */
 
 #include <vmm_error.h>
-#include <vmm_string.h>
 #include <vmm_heap.h>
 #include <vmm_stdio.h>
 #include <vmm_cmdmgr.h>
+#include <libs/stringlib.h>
 
 #define VMM_CMD_DELIM_CHAR	';'
 #define VMM_CMD_ARG_MAXCOUNT	32
@@ -39,11 +38,11 @@ struct vmm_cmdmgr_ctrl {
 
 static struct vmm_cmdmgr_ctrl cmctrl;
 
-int vmm_cmdmgr_register_cmd(vmm_cmd_t * cmd)
+int vmm_cmdmgr_register_cmd(struct vmm_cmd *cmd)
 {
 	bool found;
 	struct dlist *l;
-	vmm_cmd_t *c;
+	struct vmm_cmd *c;
 
 	if (cmd == NULL) {
 		return VMM_EFAIL;
@@ -52,8 +51,8 @@ int vmm_cmdmgr_register_cmd(vmm_cmd_t * cmd)
 	c = NULL;
 	found = FALSE;
 	list_for_each(l, &cmctrl.cmd_list) {
-		c = list_entry(l, vmm_cmd_t, head);
-		if (vmm_strcmp(c->name, cmd->name) == 0) {
+		c = list_entry(l, struct vmm_cmd, head);
+		if (strcmp(c->name, cmd->name) == 0) {
 			found = TRUE;
 			break;
 		}
@@ -65,16 +64,16 @@ int vmm_cmdmgr_register_cmd(vmm_cmd_t * cmd)
 
 	INIT_LIST_HEAD(&cmd->head);
 
-	list_add_tail(&cmctrl.cmd_list, &cmd->head);
+	list_add_tail(&cmd->head, &cmctrl.cmd_list);
 
 	return VMM_OK;
 }
 
-int vmm_cmdmgr_unregister_cmd(vmm_cmd_t * cmd)
+int vmm_cmdmgr_unregister_cmd(struct vmm_cmd *cmd)
 {
 	bool found;
 	struct dlist *l;
-	vmm_cmd_t *c;
+	struct vmm_cmd *c;
 
 	if (cmd == NULL || list_empty(&cmctrl.cmd_list)) {
 		return VMM_EFAIL;
@@ -83,8 +82,8 @@ int vmm_cmdmgr_unregister_cmd(vmm_cmd_t * cmd)
 	c = NULL;
 	found = FALSE;
 	list_for_each(l, &cmctrl.cmd_list) {
-		c = list_entry(l, vmm_cmd_t, head);
-		if (vmm_strcmp(c->name, cmd->name) == 0) {
+		c = list_entry(l, struct vmm_cmd, head);
+		if (strcmp(c->name, cmd->name) == 0) {
 			found = TRUE;
 			break;
 		}
@@ -99,11 +98,11 @@ int vmm_cmdmgr_unregister_cmd(vmm_cmd_t * cmd)
 	return VMM_OK;
 }
 
-vmm_cmd_t * vmm_cmdmgr_cmd_find(const char *cmd_name)
+struct vmm_cmd *vmm_cmdmgr_cmd_find(const char *cmd_name)
 {
 	bool found;
 	struct dlist *l;
-	vmm_cmd_t *c;
+	struct vmm_cmd *c;
 
 	if (!cmd_name) {
 		return NULL;
@@ -113,8 +112,8 @@ vmm_cmd_t * vmm_cmdmgr_cmd_find(const char *cmd_name)
 	c = NULL;
 
 	list_for_each(l, &cmctrl.cmd_list) {
-		c = list_entry(l, vmm_cmd_t, head);
-		if (vmm_strcmp(c->name, cmd_name) == 0) {
+		c = list_entry(l, struct vmm_cmd, head);
+		if (strcmp(c->name, cmd_name) == 0) {
 			found = TRUE;
 			break;
 		}
@@ -127,11 +126,11 @@ vmm_cmd_t * vmm_cmdmgr_cmd_find(const char *cmd_name)
 	return c;
 }
 
-vmm_cmd_t *vmm_cmdmgr_cmd(int index)
+struct vmm_cmd *vmm_cmdmgr_cmd(int index)
 {
 	bool found;
 	struct dlist *l;
-	vmm_cmd_t * c;
+	struct vmm_cmd *c;
 
 	if (index < 0) {
 		return NULL;
@@ -141,7 +140,7 @@ vmm_cmd_t *vmm_cmdmgr_cmd(int index)
 	found = FALSE;
 
 	list_for_each(l, &cmctrl.cmd_list) {
-		c = list_entry(l, vmm_cmd_t, head);
+		c = list_entry(l, struct vmm_cmd, head);
 		if (!index) {
 			found = TRUE;
 			break;
@@ -170,32 +169,35 @@ u32 vmm_cmdmgr_cmd_count(void)
 	return retval;
 }
 
-int vmm_cmdmgr_execute_cmd(vmm_chardev_t *cdev, int argc, char **argv)
+int vmm_cmdmgr_execute_cmd(struct vmm_chardev *cdev, int argc, char **argv)
 {
 	int ret;
-	vmm_cmd_t * cmd = NULL;
+	struct vmm_cmd *cmd = NULL;
 
 	/* Find & execute the commad */
 	if ((cmd = vmm_cmdmgr_cmd_find(argv[0]))) {
 		/* Found a matching command so execute it. */
 		if ((ret = cmd->exec(cdev, argc, argv))) {
-			vmm_cprintf(cdev, "Error %d: Command Failed\n", ret);
+			vmm_cprintf(cdev, "Error: command %s failed "
+					  "(code %d)\n", argv[0], ret);
 			return ret;
 		}
 	} else {
 		/* Did not find command. */
-		vmm_cprintf(cdev, "Unknown Command - %s\n", argv[0]);
+		vmm_cprintf(cdev, "Error: unknown command %s\n", argv[0]);
 		return VMM_ENOTAVAIL;
 	}
 
 	return VMM_OK;
 }
 
-int vmm_cmdmgr_execute_cmdstr(vmm_chardev_t *cdev, char *cmds)
+int vmm_cmdmgr_execute_cmdstr(struct vmm_chardev *cdev, char *cmds, 
+	bool (*filter)(struct vmm_chardev *cdev, int argc, char **argv))
 {
-	int argc, cmd_ret;
+	int argc, ret;
 	char *argv[VMM_CMD_ARG_MAXCOUNT];
 	char *c = cmds;
+	bool eos = 0;
 	argc = 0;
 	while (*c) {
 		while (*c == VMM_CMD_ARG_DELIM_CHAR ||
@@ -214,38 +216,47 @@ int vmm_cmdmgr_execute_cmdstr(vmm_chardev_t *cdev, char *cmds)
 		       *c != VMM_CMD_DELIM_CHAR && *c != '\0') {
 			c++;
 		}
+		if (*c == '\0')
+			eos = 1;
 		if ((*c == VMM_CMD_DELIM_CHAR || *c == '\0') && argc > 0) {
 			*c = '\0';
 			c++;
-			cmd_ret = vmm_cmdmgr_execute_cmd(cdev, argc, argv);
-			if (cmd_ret)
-				return cmd_ret;
+			if (filter && filter(cdev, argc, argv)) {
+				vmm_cprintf(cdev, "Error: command %s "
+						  "filtered\n", argv[0]);
+			} else {
+				ret = vmm_cmdmgr_execute_cmd(cdev, argc, argv);
+				if (ret)
+					return ret;
+			}
 			argc = 0;
+			if (eos)
+				break;
 		} else {
 			*c = '\0';
 			c++;
 		}
 	}
 	if (argc > 0) {
-		cmd_ret = vmm_cmdmgr_execute_cmd(cdev, argc, argv);
-		if (cmd_ret) {
-			return cmd_ret;
+		ret = vmm_cmdmgr_execute_cmd(cdev, argc, argv);
+		if (ret) {
+			return ret;
 		}
 	}
 	return VMM_OK;
 }
 
-static void cmd_help_usage(vmm_chardev_t *cdev)
+static void cmd_help_usage(struct vmm_chardev *cdev)
 {
 	vmm_cprintf(cdev, "Usage: ");
 	vmm_cprintf(cdev, "   help\n");
 	vmm_cprintf(cdev, "   help <cmd_name1> [<cmd_name2>] ...\n");
 }
 
-static int cmd_help_exec(vmm_chardev_t *cdev, int argc, char **argv)
+static int cmd_help_exec(struct vmm_chardev *cdev, int argc, char **argv)
 {
 	u32 i, cmd_count;
-	vmm_cmd_t * cmd;
+	struct vmm_cmd *cmd;
 	
 	if (argc == 1) {
 		cmd_count = vmm_cmdmgr_cmd_count();
@@ -268,7 +279,7 @@ static int cmd_help_exec(vmm_chardev_t *cdev, int argc, char **argv)
 	return 0;
 }
 
-static vmm_cmd_t help_cmd = {
+static struct vmm_cmd help_cmd = {
 	.name = "help",
 	.desc = "displays list of all commands",
 	.usage = cmd_help_usage,
@@ -277,7 +288,7 @@ static vmm_cmd_t help_cmd = {
 
 int __init vmm_cmdmgr_init(void)
 {
-	vmm_memset(&cmctrl, 0, sizeof(cmctrl));
+	memset(&cmctrl, 0, sizeof(cmctrl));
 
 	INIT_LIST_HEAD(&cmctrl.cmd_list);
 
