@@ -96,9 +96,9 @@ static struct rbd *__rbd_create(struct vmm_device *dev,
 	}
 
 	/* Setup block device instance */
-	strncpy(d->bdev->name, name, VMM_BLOCKDEV_MAX_NAME_SIZE);
+	strncpy(d->bdev->name, name, VMM_FIELD_NAME_SIZE);
 	strncpy(d->bdev->desc, "RAM backed block device", 
-		VMM_BLOCKDEV_MAX_DESC_SIZE);
+		VMM_FIELD_DESC_SIZE);
 	d->bdev->dev = dev;
 	d->bdev->flags = VMM_BLOCKDEV_RW;
 	d->bdev->start_lba = 0;
