@@ -1,6 +1,9 @@
 #ifndef _LINUX_KERNEL_H
 #define _LINUX_KERNEL_H
 
+#include <linux/err.h>
+#include <linux/bitops.h>
+
 #define ARRAY_SIZE	array_size
 
 /* FIXME: This file just a place holder for most cases */
