@@ -440,7 +440,12 @@ static int omap_uart_driver_probe(struct vmm_device *dev,
 		goto free_nothing;
 	}
 
-	strcpy(port->cd.name, dev->node->name);
+	if (strlcpy(port->cd.name, dev->node->name, sizeof(port->cd.name)) >=
+	    sizeof(port->cd.name)) {
+		rc = VMM_EOVERFLOW;
+		goto free_port;
+	}
+
 	port->cd.dev = dev;
 	port->cd.ioctl = NULL;
 	port->cd.read = omap_uart_read;
